/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 *****************************************************************************/
/**
 * @file ConfigFileImp.cpp
 * Implementation file for the libconfig concrete implementation class that
 * implements methods for reading and writing configuration information
 * from a file.
 */


/**
 * This is the concrete implementation class for gathering configuration
 * information from a file. The format of the file is expected to be
 * <Section>
 * <strKey> = <strValue> - with potentially multiple strKey/strValue pairs per strSection
 *
 * @author Dave Webster
 *
 */


//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

#include "ConfigFileImp.h"
#include <fstream>
#include "limits.h" // this will need to be defined for the UNIX env
#include "string.h"
#include "StringUtils.h"


//////////////////////////////////////////////////////////////////////
// defines
//////////////////////////////////////////////////////////////////////

#ifndef MAX_PATH
#define MAX_PATH 260
#endif

using namespace std;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/**
 * Constructor
 */
CConfigFileImp::CConfigFileImp()
{

}


/**
 * Destructor
 */
CConfigFileImp::~CConfigFileImp()
{

}

/**
 * Parse the configuration source and store the sections and 
 * strKey-strValue pairs into a hash table map.
 *
 * @param strFilename - The file name string for the configuration file to parse
 * @return 0 for success or 1 for error
 */ 
unsigned long CConfigFileImp::ReadConfig(wstring strFilename)
{
	ifstream ifs;
	string  strLine    = "";
	string  strKey     = "";
	string  strValue   = "";
	string  strSection = "";
	size_t  nPosEqual  = 0;
	unsigned long ulLineLength;
	char    cDelimiter[] = {'='};
	map<wstring,wstring>* m_pMapKeyValue = NULL;
	string strFilenameA;

	// set the member file name if needed for saving
	CConfigImp::m_szFilename = strFilename;

	// this allows for configuration files to only be in 
	// non-wide character paths.
	CStringUtils::Narrow(strFilenameA,strFilename);

	// open the file for read
	ifs.open(strFilenameA.c_str(),ios_base::in );

	// check if stream is open
	if(!ifs.is_open())
	{
		// file could not be opened
		// retrun error
		return 1;  // could not open file - some day put const in header file
	}

	// read the file into the hash tables
	while(1)
	{
		// local variables to while loop
		bool bComment = false;
		bool bSection = false;
		int  nSectionBegin = 0;
		int  nSectionEnd   = 0;
		size_t nLoop       = 0;
		size_t nKeyLength  = 0;

		// get strLine and length
		getline(ifs,strLine,'\n');

		// check if end of file
		if (( strLine.length() < 1) )
		{
			if ( ifs.eof() )
			{
				break;
			}
			else 
			{
				ifs.clear(); // blank strLine so clear bad bit so unix will get next strLine
				continue;
			}
		}

		///////////////////////////////
		// check for valid strLine length
		///////////////////////////////

		// get and check the length of strLine
		ulLineLength = strLine.length();
		if (ulLineLength > INT_MAX)
		{
			// close the file
			ifs.close();

			// inproper initialization file format
			// the strLine is longer than an integer -> hack
			return 2; // use const file in the future
		}

		// check minimum strLine lenght
		// does not make sense if < 3 characters
		if (ulLineLength < 3) 
		{
			if ( ifs.eof() )
			{
				break;
			}
			continue;
		}


		///////////////////////////////
		// check if comment strLine
		///////////////////////////////

		// scan if strLine is a comment
		for (nLoop=0; nLoop < ulLineLength - 1; ++nLoop)
		{
			switch (strLine.at(nLoop))
			{
				case ' ':
					continue;
					break;
				case ';':
				case '#':
					bComment = true;
					nLoop = ulLineLength - 1;
					break;
				default:
					bComment = false;
					nLoop = ulLineLength - 1;
					break;
			}
		}
		if (bComment == true)
		{
			if ( ifs.eof() )
			{
				break;
			}
			continue;
		}
		

		///////////////////////////////
		// get strSection strLine
		///////////////////////////////

		// check if strSection strLine
		bSection = false;
		for (nLoop=0; nLoop < ulLineLength - 2; ++nLoop)  // [ can not be last char
		{
			switch (strLine.at(nLoop))
			{
				case ' ':
					continue;
					break;
				case '[':
					bSection = true;
					nSectionBegin = nLoop;
					nLoop = ulLineLength - 2;
					break;
				default:
					bSection = false;
					nLoop = ulLineLength - 2;
					break;
			}
		}

		if (bSection == true)
		{
			bSection = false;

			for ( nLoop=ulLineLength - 1; nLoop > 0 ; --nLoop) // ] can not be first char
			{
				switch (strLine.at(nLoop))
				{
					case ' ':
						continue;
						break;
					case ']':
						bSection = true;
						nSectionEnd = nLoop;
						nLoop = 1;
						break;
					default:
						bSection = false;
						break;
				}
				if (bSection == true)
				{
						break;
				}
			}
		}
		
		//
		// if strSection is true then extract the strSection name
		//
		if (bSection ==  true)
		{
			strSection = strLine.substr ( nSectionBegin + 1, ( nSectionEnd - nSectionBegin - 1 ) ); 
		
			// make key lowercase
#if defined WIN32 && !defined UNDER_CE
			this->LowerCase(&strSection, &(ifs.getloc()));
#else
			this->LowerCase(&strSection);
#endif

			// we now have a strSection so lets add to the hash table (map)	   
			wstring strSectionW;
			CStringUtils::Widen(strSectionW, strSection);
			m_mapConfigIterator = m_mapConfig.find(strSectionW);
			if (m_mapConfigIterator == m_mapConfig.end())
			{
				// need to add to hash table
				m_pMapKeyValue = new map<wstring,wstring>;  // create empty hash table
				m_mapConfig[strSectionW] = m_pMapKeyValue;

			}
			else
			{
				// do noting it already exist in the table
				// there are two sections with the same strSection heading
			}

			continue;
		}

		//
		// check to make sure we have a strSection to put strKey/strValue pairs
		// 
		if (strSection == "")  // check to make sure there is a strSection defined prior to strKey/strValue
		{
			// malformed ini file
			continue;
		}

		///////////////////////////////
		// get the strKey strValue pairs
		///////////////////////////////

		// find the position of the equal sign
		nPosEqual = strLine.find_first_of(cDelimiter,0,sizeof(cDelimiter)); // 0 is the start pos

		// make sure that it is not at the beginning of the strLine
		// if it is at the beginning of the strLine then continue tring to 
		// parse the rest of the file and ignore the errored strLine
		if (nPosEqual == string::npos)
		{
			// malformed ini file
			continue;
		}
		strKey = strLine.substr(0, nPosEqual);
		nKeyLength = strKey.length();

		//
		// check if strSection strLine
		//
		// trim left side of strKey
		for (nLoop=0; nLoop < nKeyLength - 1; ++nLoop) 
		{
			switch (strKey.at(nLoop))
			{
				case ' ':
					continue;
					break;
				default:
					nSectionBegin = nLoop;
					nLoop = ulLineLength - 1;
					break;
			}
		}

		// trim right side of strKey
		for ( nLoop = nKeyLength - 1; nLoop > 0 ; --nLoop) 
		{
			switch (strLine.at(nLoop))
			{
				case ' ':
					continue;
					break;
				default:
					nSectionEnd = nLoop;
					nLoop = 1;
					break;
			}
		}
		strKey = strKey.substr(nSectionBegin,(nSectionEnd - nSectionBegin) + 1);
		
		// make key lowercase
		this->LowerCase(&strKey);


		///////////////////////////////
		// get value
		///////////////////////////////

		strValue = strLine.substr ( nPosEqual + 1, ( ulLineLength - nPosEqual - 1 ) );
		CStringUtils::Trim(strValue);

		wstring strSectionW;
		CStringUtils::Widen(strSectionW, strSection); 
		
		// add strKey and strValue to hash table (map)
		m_mapConfigIterator = m_mapConfig.find(strSectionW); //find the correct strSection

		if (m_mapConfigIterator != m_mapConfig.end())    // better always be the case
		{
			wstring wKey, wValue;
			CStringUtils::Widen(wKey,strKey);
			CStringUtils::Widen(wValue,strValue);
			// need to add to hash table
			(*(*m_mapConfigIterator).second)[wKey] = wValue;

		}

		//
		// check if end of file - not really needed
		//
		if (ifs.eof())
		{
			break;
		}
	}

	// close the input file
	ifs.close();
	 
	// successful
	return 0;
}


/**
 * Save the information in the table out to a file
 *
 * @param strFilenameA A wide string containing the file name
 *                    to save the file.
 *
 * @return:	0 for success and >0 for failure
 *
 * @notes This function has only real significance to config file parsing.
 *        It saves the configuration file output after normalization
 *        to another file.
 */
unsigned long CConfigFileImp::SaveConfig(wstring strFilename)
{
	// locals
	ofstream ofs;
	list<wstring> listsec;
	list<wstring> listkey;
	list<wstring>::iterator itSec;
	list<wstring>::iterator itKey;
	string szFileName;

	// Get a narrow string from the wide string passed into us.
	CStringUtils::Narrow(szFileName,strFilename);
	
	// open the file for writing
	ofs.open(szFileName.c_str(),ios::out);

	// check if the file opened
	if (!ofs.is_open())
		return 1;

	// Get a list of the section names from the hash table.
	listsec = this->GetSectionNames();
	
	// Iterate through the list we just got.
	for (itSec = listsec.begin(); itSec != listsec.end(); itSec++)
	{
		string strSection;
		CStringUtils::Narrow(strSection,*itSec);

		// print strSection name
		ofs << "[" << strSection << "]" << endl;

		// Get a list of the keys from the specified section.
		listkey = this->GetKeys(*itSec);

		// Iterate through the keys that we just obtained.
		for (itKey = listkey.begin(); itKey != listkey.end(); itKey++)
		{
			string szKey;
			string szValue;
			wstring wszValue;

			CStringUtils::Narrow(szKey, *itKey);

			// Get the value of the key specified.
			wszValue = this->GetValue(*itSec,*itKey);

			CStringUtils::Narrow(szValue,wszValue);

			ofs << szKey << "=" << szValue << endl;
		}

		ofs << endl;
	}

	// close the file
	ofs.close();

	return 0;
}
