/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 *****************************************************************************/
/**
 * @file ConfigRegImp.cpp
 * Implementation file for the libconfig concrete implementation that
 * implements methods for reading and writing configuration information from
 * the windows registry.
 */


/**
 * This is the concrete implementation class for gathering configuration
 * information from the windows registry. The format of the registry is
 * expected to be
 * <Subkey>
 * <data name> = <data value> - with potentially multiple name/value pairs
 *								per subkey.
 * Keys under a subkey results in an error, the root key to search for is
 * expected to have subkeys only one level deep.
 *
 */


//////////////////////////////////////////////////////////////////////
// includes
//////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <regstr.h>
#include "ConfigRegImp.h"
#include "StringUtils.h"


///////////////////////////////////////////////////////////////////////////////
// namespace
///////////////////////////////////////////////////////////////////////////////

using namespace std;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
/**
 * Constructor
 */
CConfigRegImp::CConfigRegImp()
{

}


/**
 * Destructor
 */
CConfigRegImp::~CConfigRegImp()
{

}


/** 
 * Parse the configuration source and store the sections/subkeys
 * and key-value/value name-value data pairs into a hash table 
 * map.
 * 
 * @param strFilename The file name wide string or the root key 
 *                    wide string to parse
 * 
 * @return	0 for success and >0 for error
 */
unsigned long CConfigRegImp::ReadConfig(wstring szhKey)
{
	wstring key     = L"";
	wstring	newkey	= L"";  // rename the key if it contains spaces.
	wstring value   = L"";
	wstring data	= L"";
	wstring	tempkey = L"";
	
	
	HKEY			hRegKey        = NULL;
	HKEY			hRoot          = NULL;
	HKEY            hRegSubKey     = NULL;
	HKEY            hRegNewSubKey  = NULL;
	KEY2VALUEMAP*	m_pMapKeyValue = NULL;
	DWORD			dwIndex = 0;
	DWORD			dwNumberofValues = 0;
	DWORD			dwNumberofKeys = 0;
	DWORD			dwType = 0;
	wchar_t			delimiter[] = {L"\\"};
	int				iposFirstSlash = 0;
	int				iposLastSlash = 0;
	int				iKeyLength = 0;
	bool			bTruncate = false;
	bool			bNewKey = false;

	// Parse the string. Determine if it contains the root key
	iposFirstSlash = szhKey.find_first_of ( delimiter, 0, sizeof ( delimiter ) );

	if ( iposFirstSlash == 0 )
	{
		// Malformed string. A registry key cannot start with a slash
		return 1;
	}
	
	// Find the root path. Set the root path if it exists within the string
	// Otherwise assume that it's a key under HKEY_LOCAL_MACHINE
	key = szhKey.substr ( 0, iposFirstSlash );
	iKeyLength = key.length();

	if ( key.compare ( 0, iKeyLength + 1, L"HKEY_LOCAL_MACHINE" ) == 0 )
	{
		hRoot = HKEY_LOCAL_MACHINE;
		bTruncate = true;
	}
	else
		if ( key.compare ( 0, iKeyLength + 1, L"HKEY_USERS" ) == 0 )
		{
			hRoot = HKEY_USERS;
			bTruncate = true;
		}
		else
			if ( key.compare ( 0, iKeyLength + 1, L"HKEY_CURRENT_CONFIG" ) == 0 )
			{
				hRoot = HKEY_CURRENT_CONFIG;
				bTruncate = true;
			}
			else
				if ( key.compare ( 0, iKeyLength + 1, L"HKEY_CLASSES_ROOT" ) == 0 )
				{
					hRoot = HKEY_CLASSES_ROOT;
					bTruncate = true;
				}
				else
					if ( key.compare ( 0, iKeyLength + 1, L"HKEY_CURRENT_USER" ) == 0 )
					{
						hRoot = HKEY_CURRENT_USER;
						bTruncate = true;
					}
					else
						hRoot = HKEY_LOCAL_MACHINE;

	// Create a const char* subkey based on whether the root name is included in the key.
	if ( bTruncate )
	{
		// Truncate from the slash to the end
		szhKey = szhKey.substr ( iposFirstSlash + 1, ( szhKey.length() - iposFirstSlash ) + 1 );
	}

	// To determine if the key exists and to get a handle on the key, open it

	long lResult = ::RegOpenKeyExW (	
									hRoot,				// handle to open key
									szhKey.c_str(),		// address of name of subkey to open
									0,					// reserved
									KEY_READ,			// security access mask
									&hRegKey			// address of handle to open key
								   );

	if ( lResult != ERROR_SUCCESS )
	{
		DWORD dwError = ::GetLastError();

		// The key does not exist or the user does not have access rights
		return 1;
	}

	// Determine information about the Key
	// Important information culled is the number of value entries and the longest value
	// name length and longest value data length

	lResult = ::RegQueryInfoKeyW(	
									hRegKey,			// The key being examined
									NULL,				// Not implemented
									NULL,				// size of buffer for class string
									NULL,				// reserved
									&dwNumberofKeys,	// number of subkeys
									NULL,				// longest subkey name length
									NULL,				// longest class string length
									NULL,				// number of value entries
									NULL,				// longest value name length
									NULL,				// longest value data length
									NULL,				// length of the security descriptor
									NULL 				// last time the key was modified
								);

	// This is a major decision point. We can either force the user to enter 
	// A registry key containing sub keys, or allow them to enter the subkey and
	// treat it as the only 'section'. I've decided upon the former - BS 11/10/99

	if ( lResult != ERROR_SUCCESS || dwNumberofKeys == 0 )
	{
		// Could not obtain the key information.
		// Or the original key entered is a subkey
		// We should split this up for error handling purposes
		::RegCloseKey ( hRegKey );
		return 1;
	}

	// Iterate through all the subkeys

	for ( DWORD j = 0; j < dwNumberofKeys; ++j )
	{
		wchar_t	szKey[REGSTR_MAX_VALUE_LENGTH];
		DWORD	cbKey = sizeof ( szKey );


		lResult = ::RegEnumKeyExW (
									hRegKey,	// handle to key to enumerate
									j,			// index of subkey to enumerate
									szKey,		// address of buffer for subkey name
									&cbKey,		// address for size of subkey buffer
									NULL,		// reserved
									NULL,		// address of buffer for class string
									NULL,		// address for size of class buffer
									NULL		// address for time key last written to
								 );

		if ( lResult != ERROR_SUCCESS )
		{
			// Could not obtain a list of subkeys from the main key
			::RegCloseKey ( hRegKey );
			return 1;
		}

		// Find the last folder level to serve as the 'section' header
		// The decision was made that the last folder would represent the section
		// header. The name returned by RegEnumKeyEx is this folder
		key = szKey;
		int iKeyLength = key.length();
		int iSectionBegin, iSectionEnd;
		
		// Remove tailing and leading spaces from the key.
		// trim left side of key
		for ( int left = 0; left < iKeyLength; ++left ) 
		{
			switch ( key.at ( left ) )
			{
				case L' ':
					continue;
					break;
				default:
					iSectionBegin = left;
					left = iKeyLength;
					break;
			}
		}
		// trim right side of key
		for ( int right = iKeyLength - 1; right > 0 ; --right ) 
		{
			switch ( key.at ( right ) )
			{
				case L' ':
					continue;
					break;
				default:
					iSectionEnd = right;
					right = 1;
					break;
			}
		}
		newkey = key.substr ( iSectionBegin, ( iSectionEnd - iSectionBegin ) + 1 );
		
		// check to see if the key has no spaces, if it does we need to save the information to the new key.
		if ( newkey.compare ( key ) != 0 )
			bNewKey = true;

		// make key lowercase
		this->LowerCase ( &newkey );

		// we now have a key so lets add to the hash table (map)
		m_mapConfigIterator = m_mapConfig.find ( newkey );
		if (m_mapConfigIterator == m_mapConfig.end())
		{
			// iterator reached end of hash table without finding the key
			// need to add key to hash table
			m_pMapKeyValue = new KEY2VALUEMAP;  // create empty hash table
			m_mapConfig[newkey] = m_pMapKeyValue;
		}
		
		// Create a temporary key to modify each time through the loop.
		tempkey = szhKey;
		tempkey.append ( L"\\" );
		tempkey.append ( key );

		// Now open the subkey(s)
		lResult = ::RegOpenKeyExW (
									hRoot,				// handle to open key
									tempkey.c_str(),	// address of name of subkey to open
									0,					// reserved
									KEY_READ,		// security access mask
									&hRegSubKey			// address of handle to open key
					  			 );

		if ( lResult != ERROR_SUCCESS )
		{
			// The subkey does not exist or the user does not have access rights
			::RegCloseKey ( hRegKey );
			return 1;
		}

		// If we need to create a new key, do so now
		if ( bNewKey )
		{
			DWORD	dwKeyCreated;

			lResult = ::RegCreateKeyExW	(	
											hRegKey,					// handle to open key
											newkey.c_str(),				// subkey name
											0,							// reserved
											NULL,						// class string
											REG_OPTION_NON_VOLATILE,    // special options flag
											KEY_WRITE,					// desired security access
											NULL,
											&hRegNewSubKey,				// receives opened handle
											&dwKeyCreated				// disposition value buffer
										);

			if ( lResult != ERROR_SUCCESS )
			{
				// Was not able to create a new key or open a key with the new name
				::RegCloseKey ( hRegSubKey );
				::RegCloseKey ( hRegKey );
				return 1;
			}
		}

		// Determine information about the subkey
		// Important information culled is the number of value entries

		lResult = ::RegQueryInfoKeyW(
										hRegSubKey,			// The key being examined
										NULL,				// Not implemented
										NULL,				// size of buffer for class string
										NULL,				// reserved
										NULL,				// number of subkeys
										NULL,				// longest subkey name length
										NULL,				// longest class string length
										&dwNumberofValues,	// number of value entries
										NULL,				// longest value name length
										NULL,				// longest value data length
										NULL,				// length of the security descriptor
										NULL 				// last time the key was modified
									);

		if ( lResult != ERROR_SUCCESS )
		{
			// Could not obtain the subkey information.
			// It is not an error if this subkey has no values, it's just an empty hash
			// We should split this up for error handling purposes
			if ( bNewKey )
			{
				::RegCloseKey ( hRegNewSubKey );
			}
			::RegCloseKey ( hRegSubKey );
			::RegCloseKey ( hRegKey );
			return 1;
		}

		// Now iterate through the values to obtain the mapping
		for ( DWORD i = 0; i < dwNumberofValues; ++i )
		{

			wchar_t	szValue[REGSTR_MAX_VALUE_LENGTH]; // SDK declares 256 to be a reasonable limit
			wchar_t	szData[REGSTR_MAX_VALUE_LENGTH];  // SDK declares 256 to be a reasonable limit
			DWORD	cbValue = sizeof ( szValue );
			DWORD	cbData = sizeof ( szData );

			lResult = ::RegEnumValueW ( 
										hRegSubKey,			// handle to key to query
										i,					// index of value to query
										szValue,			// buffer for value string
										&cbValue,			// size of value buffer
										NULL,				// reserved
										&dwType,			// buffer for type code
										(LPBYTE)&szData,	// buffer for value data
										&cbData				// size of data buffer
									  );

			if ( lResult != ERROR_SUCCESS )
			{
				// Could not enumerate the registry key value/data items.
				if ( bNewKey )
				{
					::RegCloseKey ( hRegNewSubKey );
				}
				::RegCloseKey ( hRegSubKey );
				::RegCloseKey ( hRegKey );
				return 1;
			}

			value = szValue;
			iKeyLength = value.length();

			// Remove tailing and leading spaces from the key.
			// trim left side of key
			for ( int left = 0; left < iKeyLength; ++left ) 
			{
				switch ( value.at ( left ) )
				{
					case L' ':
						continue;
						break;
					default:
						iSectionBegin = left;
						left = iKeyLength;
						break;
				}
			}
			// trim right side of key
			for ( int right = iKeyLength - 1; right > 0 ; --right ) 
			{
				switch ( value.at ( right ) )
				{
					case L' ':
						continue;
						break;
					default:
						iSectionEnd = right;
						right = 1;
						break;
				}
			}

			value = value.substr ( iSectionBegin, ( iSectionEnd - iSectionBegin ) + 1 );

			// Ignore default values for keys.
			if ( value.compare ( L"" ) == 0 )
				continue;

			// make key lowercase
			this->LowerCase ( &value );
			data = szData;

			// Save the information to the appropriate key

			if ( bNewKey )
			{
				lResult = ::RegSetValueExW ( 
											hRegNewSubKey,        // handle to key to set value for
											value.c_str(),		// the value to set
											0,					// reserved
											REG_SZ,				// flag for value type
											(LPBYTE)szData,		// address of value data
											cbData				// size of value data
										   );

			}
			else
			{
				lResult = ::RegSetValueExW ( 
											hRegSubKey,			// handle to key to set value for
											value.c_str(),		// the value to set
											0,					// reserved
											REG_SZ,				// flag for value type
											(LPBYTE)szData,		// address of value data
											cbData				// size of value data
										   );
			}

			// add key and value to hash table (map)
			m_mapConfigIterator = m_mapConfig.find ( newkey );  // find the correct key
			if (m_mapConfigIterator != m_mapConfig.end())    // better always be the case
			{
				// iterator reached end of hash table without finding the key
				// need to add to hash table
				(*(*m_mapConfigIterator).second)[value] = data;

			}

		} // End iterating through the values
		if ( bNewKey )
		{
			::RegCloseKey ( hRegNewSubKey );
		}

		::RegCloseKey ( hRegSubKey );

	} // End for loop for the sub keys


	::RegCloseKey ( hRegKey );
	return 0;

}