/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file
 * Implementation file for CApexException class which implements the methods
 * used for exceptions.
 */


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////
#include "ApexException.h"
#include <sstream>
#include <vector>
#include <stdlib.h>
#include <string.h>

#ifndef UNDER_CE
	#include <errno.h>
#endif

#ifdef _MSC_VER //support GetSystemError() function
#  include <windows.h>
#endif


///////////////////////////////////////////////////////////////////////////////
// using namespace
///////////////////////////////////////////////////////////////////////////////
using namespace std;


///////////////////////////////////////////////////////////////////////////////
// portability functions
///////////////////////////////////////////////////////////////////////////////

//Defines secure versions of standard C functions.
//Wrapped by a test to see if they're already defined.
#if !defined(__STDC_SECURE_LIB__) || defined(UNDER_CE)

	#ifndef EINVAL
	#define EINVAL 22
	#endif

	typedef int errno_t;

	static inline errno_t mbstowcs_s(
		size_t *pReturnValue, //The number of characters converted.
		wchar_t *wcstr, //Address of buffer for the resulting converted wide character string.
		size_t sizeInWords, //The size of the wcstr buffer in words.
		const char *mbstr, //The address of a sequence of null terminated multibyte characters.
		size_t count ) //The maximum number of wide characters to store in the wcstr buffer, not including the terminating null
	{
		if((NULL == pReturnValue) || (NULL == wcstr) || (NULL == mbstr) ||
			(sizeInWords == 0))
		{
			return EINVAL;
		}
		*pReturnValue = mbstowcs( wcstr, mbstr, min<size_t>( sizeInWords - 1, count ));
		return 0;
	}

#endif



///////////////////////////////////////////////////////////////////////////////
// static functions
///////////////////////////////////////////////////////////////////////////////


/**
 * Convert narrow string to wide string.
 *
 * This code exists elsewhere in a utility library, but is duplicated here so that
 * this exception library can stand on its own, with no dependencies on other libs.
 *
 * @param strIn - const string& is the input string.
 * @param strOut - const wstring& is the output string.
 */
static void Widen( wstring& strOut, const char* strIn, size_t nInLen )
{
#if defined UNDER_CE
	int nResult = 0;

	// get the lenght of the string
	int nLen = ::MultiByteToWideChar(CP_ACP, 0, strIn, -1, NULL, NULL);

	// allocate space for the wide character string
	wchar_t* pcOut = new wchar_t[nLen+1];

    // Covert to Unicode.
	nResult = MultiByteToWideChar(CP_ACP, 0,  strIn, -1, pcOut, nLen);	

	if (nResult == 0)
	{
		strOut = L"";
	}
	else
	{
		strOut = pcOut;
	}

	delete[] pcOut;
#else
	vector<wchar_t> vecOut( nInLen + 1, L'\0' ); //+1 for terminating '\0'
	size_t nCharsConverted = 0;

	//possible error conditions are handled implicitly
	mbstowcs_s( &nCharsConverted, &vecOut[0], vecOut.size(), strIn, nInLen );

	//mbstowcs_s always null-terminates the dest string, even if there's an error.
	//so this assignment is safer than using strOut.assign( vecOutW.begin(), vecOutW.end())
	strOut.reserve( nCharsConverted + 1 );
	strOut = &vecOut[0];
#endif
}


///////////////////////////////////////////////////////////////////////////////
// method definitions
///////////////////////////////////////////////////////////////////////////////

/**
 * Constructor
 */
CApexException::CApexException()
{
	// Set the default values for an exception.
	this->m_nLineNumber    = -1;
	this->m_strDescription = L"Unknown Error";
	this->m_strFile        = L"Unknown File";
	this->m_enumSeverity   = ERROR_FATAL;
}


/**
 * Constructor
 *
 * @param enumSeverity An EnumSeverity value indicating the the severity of the exception.
 * @param strDescription A reference to a std::wstring that holds the description of the exception.
 * @param strFile A reference to a std::wstring that holds the filename where the exception happened. Use the macro __FILE__
 * @param nLineNumber A long that holds the line number in the file where the exception happened. Use the macro __LINE__
 */
CApexException::CApexException( EnumSeverity enumSeverity, const wstring& strDescription, const string& strFile, long nLineNumber )
{
	this->m_nLineNumber    = nLineNumber;
	this->m_enumSeverity   = enumSeverity;
	this->m_strDescription = strDescription;
	SetFile(strFile);
}


/**
 * Constructor
 *
 * @param enumSeverity An EnumSeverity value indicating the the severity of the exception.
 * @param strDescription A reference to a std::wstring that holds the description of the exception.
 * @param strFile A reference to a std::string that holds the filename where the exception happened. Use the macro __FILE__
 * @param nLineNumber A long that holds the line number in the file where the exception happened. Use the macro __LINE__
 */
CApexException::CApexException( EnumSeverity enumSeverity, const string& strDescription, const string& strFile, long nLineNumber )
{
	this->m_nLineNumber    = nLineNumber;
	this->m_enumSeverity   = enumSeverity;
	SetDescription(strDescription);
	SetFile(strFile);
}


/**
 * Copy constructor for copying an exception
 */
CApexException::CApexException(const CApexException& exception)	
{
	this->operator=( exception );
}


/**
 * Destructor
 */
CApexException::~CApexException()
{
}


/**
 * This method sets the filename that caused the error. This method ensures
 * that the filename stored is actually a wide string.
 *
 * @param strFile A reference to a std::string that holds the filename where the error occurred.
 */
void CApexException::SetFile(const string& strFile)
{
	wstring strFileW;

	// Make sure the file name is wide string.
	Widen( strFileW, strFile.c_str(), strFile.size());

	// A swap is cheaper than a copy.
	this->m_strFile.swap( strFileW );
}


/**
 * This method sets the filename that caused the error.
 *
 * @param strFile A reference to a std::wstring that holds the filename where the error occurred.
 */
void CApexException::SetFile(const wstring& strFile)
{
	this->m_strFile = strFile;
}


/**
 * This method sets the description of the error. This method ensures that the
 * description stored is actually a wide string.
 *
 * @param strDescription A reference to a std::string that holds the desciption of the error.
 */
void CApexException::SetDescription(const string& strDescription)
{
	wstring strDescriptionW;

	// Convert the description to a wide string.
	Widen( strDescriptionW, strDescription.c_str(), strDescription.size());

	// A swap is cheaper than a copy.
	this->m_strDescription.swap( strDescriptionW );
}


/**
 * This method sets the description of the error.
 *
 * @param strDescription A reference to a std::wstring that holds the desciption of the error.
 */
void CApexException::SetDescription(const wstring& strDescription)
{
	this->m_strDescription = strDescription;
}


/**
 * This method retrieves the description of the exception.
 *
 * @param bAddFileName A boolean indicating if the filename and line number added to the descripton. The default is true.
 */
wstring CApexException::GetDescription(bool bAddFileName) const
{
	// Should the filename information be added?
	if (false == bAddFileName)
	{
		// Caller just wants the description string
		return m_strDescription;
	}

	// Caller wants file info added to it
	wstringstream strDescription; //stream used to format
	strDescription << m_strDescription << L" ("
		<< GetFileName() << L":"
		<< m_nLineNumber << L")";

	return strDescription.str();
}


/**
 * This method gets the filename without the path where the error occurred.
 *
 * @return A std:wstring containing just the filename.
 */
wstring CApexException::GetFileName() const
{
	// Find the last slash
	const size_t nPos = m_strFile.find_last_of(L'\\');

	// If no slash found, just return the whole file
	if (nPos == string::npos)
	{
		return m_strFile;
	}

	// Get just the file name
	const wstring strReturn = m_strFile.substr(nPos+1, m_strFile.length() - nPos);

	// Return the file name
	return strReturn;
}


#ifdef _MSC_VER //this function only available on Windows
/**
 * Returns the string from a system exception.
 * 
 * @return The string from the system based on the most recent system error number.
 */
wstring CApexException::GetSystemError()
{
	// Retrieve the last error from the system.
	const DWORD nErrorNumber = ::GetLastError();

	LPVOID lpMsgBuf = NULL;
	DWORD  nResult  = 0;

	// Retrieve the message description from the system for the specified error number.
	nResult = FormatMessageW(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
							 NULL, nErrorNumber,
							 MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
							 (wchar_t*)&lpMsgBuf,
							 0, NULL);

	// Did we get a message string back?
	if (nResult == 0)
	{
		// If not, format our own.
		wchar_t cBuf[16] = {'\0'};

		// Convert the error number to a wide string.
		_ltow_s(nErrorNumber, cBuf, sizeof(cBuf), 10);

		// Format our error message.
		wstring strErr = L"System error (";

		// Add the error number.
		strErr += cBuf;
		strErr += L").";

		// Return the error message.
		return strErr.c_str();
	}

	wstring strErr = (wchar_t*)lpMsgBuf;

	// Add a newline to our string.
	strErr += L"\n";

	// Free any memory allocated when we retrieve the error message.
	LocalFree(lpMsgBuf);

	// Return the error message.
	return strErr;
}
#endif


/**
 * = operator.  
 *
 * @param rhs - CApexException that is the right hand side of the = sign.
 */
CApexException& CApexException::operator=( const CApexException& rhs )
{
	this->m_enumSeverity   = rhs.GetSeverity();
	this->m_nLineNumber    = rhs.GetLineNumber();
	this->m_strDescription = rhs.GetDescription(false);
	this->m_strFile        = rhs.GetFilePath();
	return *this;
}


/**
 * This method retrieves the severity of the exception.
 *
 * @return An EnumSeverity value specifying the severity of the exception.
 */
EnumSeverity CApexException::GetSeverity() const
{
	return m_enumSeverity;
}


/**
 * This method retrieves the error line number.
 *
 * @return A long value indicating the line number in the file where the error occurred.
 */
long CApexException::GetLineNumber() const
{
	return this->m_nLineNumber;
}


/**
 * This method retrieves the filename and path in which the error occurred.
 *
 * @return A wstring holding the fully qualified filename in which the error occurred.
 */
const wstring& CApexException::GetFilePath() const
{
	return this->m_strFile;
}

