/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file FileUtils.h
 * Header file for the CFileUtils class which defines helper file utility
 * methods.
 */


///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////
#pragma once


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////
#include <string>
#include <vector>
#include <limits>
#include <stdlib.h>

#ifdef max
	#undef max
#endif

/**
 * This class implements basic file helper methods.
 */
namespace CFileUtils
{
	/**
	* This method retrieves the path to the current executable. This will not
	* include the last \.
	* 
	* @return A std::wstring that contains the path to the current running executable.
	*/
	std::wstring GetCurrentAppPath();

	/**
	* This method deletes the file passed in.
	*
	* @param A const std::wstring holding the file that is to be deleted.
	*/
	void FileDelete( const std::wstring& strFile );

	/**
	* This method sets the attributes of the file passed in to the attributes
	* pass in as well.
	*
	* @param strFile A reference to a std::string that holds the filename of the file to be altered.
	* @param nFileAttributes An unsigned long holding the file attributes to be applied to the file.
	*
	* @notes A single attribute or multiple attributes can be applied to the file
	*        with a single call to the method. When specifying more than one
	*        attribute, you should bitwise OR the attributes.
	*
	*        Such As: SetFileAttributes(strFile, FILE_ATTRIBUTE_READONLY);
	*                 SetFileAttributes(strFile, FILE_ATTRIBUTE_READONLY | FILE_ATTRIBUTE_HIDDEN);
	*
	*        The file attributes available to you are:
	*           FILE_ATTRIBUTE_ARCHIVE
	*           FILE_ATTRIBUTE_HIDDEN
	*           FILE_ATTRIBUTE_NORMAL
	*           FILE_ATTRIBUTE_NOT_CONTENT_INDEXED
	*           FILE_ATTRIBUTE_OFFLINE
	*           FILE_ATTRIBUTE_READONLY
	*           FILE_ATTRIBUTE_SYSTEM
	*           FILE_ATTRIBUTE_TEMPORARY
	*/
	void SetFileAttributes( const std::string& strFile, unsigned long nFileAttributes );

	/**
	* This method sets the attributes of the file passed in to the attributes
	* pass in as well. (Wide String Version)
	*
	* @param strFile A reference to a std::wstring that holds the filename of the file to be altered.
	* @param nFileAttributes An unsigned long holding the file attributes to be applied to the file.
	*
	* @notes A single attribute or multiple attributes can be applied to the file
	*        with a single call to the method. When specifying more than one
	*        attribute, you should bitwise OR the attributes.
	*
	*        Such As: SetFileAttributes(strFile, FILE_ATTRIBUTE_READONLY);
	*                 SetFileAttributes(strFile, FILE_ATTRIBUTE_READONLY | FILE_ATTRIBUTE_HIDDEN);
	*
	*        The file attributes available to you are:
	*           FILE_ATTRIBUTE_ARCHIVE
	*           FILE_ATTRIBUTE_HIDDEN
	*           FILE_ATTRIBUTE_NORMAL
	*           FILE_ATTRIBUTE_NOT_CONTENT_INDEXED
	*           FILE_ATTRIBUTE_OFFLINE
	*           FILE_ATTRIBUTE_READONLY
	*           FILE_ATTRIBUTE_SYSTEM
	*           FILE_ATTRIBUTE_TEMPORARY
	*/
	void SetFileAttributes( const std::wstring& strFile, unsigned long nFileAttributes );

	/**
	* This method makes sure the file passed in is writable. (Wide String Version)
	*
	* @param strFile A const std::wstring containing the filename to be made writeable.
	*/
	void SetFileWriteable( const std::wstring& strFile );

	/**
	* This method makes sure the file passed in is writable.
	*
	* @param strFile A const std::string containing the filename to be made writeable.
	*/
	void SetFileWriteable( const std::string& strFile );

	/**
	* This method makes sure the file passed in is read-only. (Wide String Version)
	*
	* @param strFile A const std::wstring containing the filename to be made read-only.
	*/
	void SetFileReadonly( const std::wstring& strFile );

	/**
	* This method makes sure the file passed in is read-only.
	*
	* @param strFile A const std::string containing the filename to be made read-only.
	*/
	void SetFileReadonly( const std::string& strFile );

	/**
	* This method determines if a file exists or not. (Wide String Version)
	*
	* @param strFile A const std::wstring containing the file that is to be checked.
	*
	* @return A bool indicating true if the file exists, otherwise false.
	*/
	bool FileExists( const std::wstring& strFile );

	/**
	* This method determines if a file exists or not.
	*
	* @param strFile A const std::wstring containing the file that is to be checked.
	*
	* @return A bool indicating true if the file exists, otherwise false.
	*/
	bool FileExists( const std::string& strFile );

	/**
	* This method retrieves the size of the file (in bytes) that is passed in.
	* (Wide String Version)
	*
	* @param strFile A const std::wstring holding the name of the file to check.
	*
	* @return An unsigned long value indicating the size of the file in bytes.
	*/
	unsigned long FileGetSize( const std::wstring& strFile );

	/**
	* This method retrieves the size of the file (in bytes) that is passed in.
	*
	* @param strFile A const std::string holding the name of the file to check.
	*
	* @return An unsigned long value indicating the size of the file in bytes.
	*/
	unsigned long FileGetSize( const std::string& strFile );

	/**
	* This method reads data from the file passed in and returns the data in a
	* vector of unsigned char(s). (Wide String Version)
	*
	* @param strFile A const std::wstring holding the name of the file that data will be read from.
	* @param nMinReadSize A size_t that holds the minimum size of the file to be read.
	* @param nMaxReadSize A size_t that holds the maximum size of the file to be read.
	*
	* @return A vector of unsigned char(s) that holds the data read from the file.
	*/
	std::vector<unsigned char> FileRead( const std::wstring& strFile, size_t nMinReadSize = 0, size_t nMaxReadSize = std::numeric_limits<size_t>::max());

	/**
	* This method reads data from the file passed in and returns the data in a
	* vector of unsigned char(s).
	*
	* @param strFile A const std::string holding the name of the file that data will be read from.
	* @param nMinReadSize A size_t that holds the minimum size of the file to be read.
	* @param nMaxReadSize A size_t that holds the maximum size of the file to be read.
	*
	* @return A vector of unsigned char(s) that holds the data read from the file.
	*/
	std::vector<unsigned char> FileRead( const std::string& strFile, size_t nMinReadSize = 0, size_t nMaxReadSize = std::numeric_limits<size_t>::max());

	/**
	* This method writes data to the filename passed into it. (Wide String Version)
	*
	* @param strFile A const std::wstring that holds the filename to be written to.
	* @param vecData A vector containing the data (unsigned char) to be written to the file.
	* @param bOverwrite A boolean indicating whether we should overwrite a file if it already exists.
	*
	* @notes This method will only write data to the file if it does not already
	*        exist or if the boolean parameter is set to true. If the file already
	*        exists then an exception will be thrown.
	*/
	void FileWrite( const std::wstring& strFile, std::vector<unsigned char> vecData, bool bOverwrite = true );

	/**
	* This method writes data to the filename passed into it.
	*
	* @param strFile A const std::string that holds the filename to be written to.
	* @param vecData A vector containing the data (unsigned char) to be written to the file.
	* @param bOverwrite A boolean indicating whether we should overwrite a file if it already exists.
	*
	* @notes This method will only write data to the file if it does not already
	*        exist or if the boolean parameter is set to true. If the file already
	*        exists then an exception will be thrown.
	*/
	void FileWrite( const std::string& strFile, std::vector<unsigned char> vecData, bool bOverwrite = true );

	/**
	* This method will compare two files and indicate if both the files are the same.
	* (Wide String Version)
	*
	* @param strFileOne A const std::wstring that holds the filename of the first file to compare.
	* @param strFileTwo A const std::wstring that holds the filename of the second file to compare.
	* @param pnPos A pointer to a size_t which will return the byte position of where the files begin to differ.
	*
	* @return A boolean indicating if the two files are the same. True if they are the same, otherwise false.
	*
	* @notes If the files are the same the position pointer will point to the last byte in the file.
	*/
	bool FileCompare( const std::wstring& strFileOne, const std::wstring& strFileTwo, size_t *pnPos );

	/**
	* This method will compare two files and indicate if both the files are the same.
	*
	* @param strFileOne A const std::string that holds the filename of the first file to compare.
	* @param strFileTwo A const std::string that holds the filename of the second file to compare.
	* @param pnPos A pointer to a size_t which will return the byte position of where the files begin to differ.
	*
	* @return A boolean indicating if the two files are the same. True if they are the same, otherwise false.
	*
	* @notes If the files are the same the position pointer will point to the last byte in the file.
	*/
	bool FileCompare( const std::string& strFileOne, const std::string& strFileTwo, size_t *pnPos );
}

