/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 *
 *****************************************************************************/
/**
 * @file
 * Implementation file for CStringUtils class which is a utility class that
*  contains std::string methods that are commonly used.
 */


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////

#include "StringUtils.h"
#include "CrtSecureFunctions.h"
#include <vector>
#include <cctype> // for toupper
#include <algorithm>

#if defined UNDER_CE
#include <windows.h>
#endif


///////////////////////////////////////////////////////////////////////////////
// namespace
///////////////////////////////////////////////////////////////////////////////

using namespace std;


const string CStringUtils::strDefaultStrip = string( 1, '\0' ) + " \t\r\n\v\f";


/**
* This will trim the left side of a string.
*
* @param strInput - This is the input string to trim.  
* @param strStrip - This is a parameter that describes the characters to strip.
*                   By default if this parameter is not set it will be set to 
*                   strDefaultStrip.  This will strip all white space.
*
* @return A std::string containing the original string with the left side stripped.
*/
string CStringUtils::TrimLeft(const string& strInput, const string& strStrip)
{
	// Set the initial value
	string strReturn = strInput;

	// Trim the left side
	strReturn.erase(0, strReturn.find_first_not_of(strStrip));
	
	// Return the trimmed string
	return strReturn;
}


/**
* This will trim the left side of a string.
*
* @param strInput - This is the input string to trim.  
* @param strStrip - This is a parameter that describes the characters to strip.
*                   By default if this parameter is not set it will be set to 
*                   strDefaultStrip.  This will strip all white space.
*
* @return A std::string containing the original string with the right side stripped.
*/
string CStringUtils::TrimRight(const string& strInput, const string& strStrip)
{
	// Set the initial value
	string strReturn = strInput;

	// Trim the right side
	strReturn.erase(strReturn.find_last_not_of(strStrip) + 1);

	// Return the trimmed string
	return strReturn;
}


/**
* This will trim the left and right side of a string.
*
* @param strInput - This is the input string to trim.  
* @param strStrip - This is a parameter that describes the characters to strip.
*                   By default if this parameter is not set it will be set to 
*                   strDefaultStrip.  This will strip all white space.
*
* @return A std::string containing the original string with the left and right
*         sides stripped.
*/
string CStringUtils::Trim(const string& strInput, const string& strStrip)
{
	if( strInput.empty()) return strInput; //nothing to trim

	//find first character of substring
	const string::size_type nLeftStart = strInput.find_first_not_of(strStrip);

	//might be that the string is made entirely of strip characters
	if( nLeftStart >= strInput.length()) return ""; //trim it all!

	//find last character of substring
	const string::size_type nRightEnd = strInput.find_last_not_of(strStrip);

	//return substring
	return strInput.substr( nLeftStart, nRightEnd + 1 );
}


/**
* This will trim the left and right side of a string.
*
* @param str String to be trimmed.
* @param strStrip - This is a parameter that describes the characters to strip.
*                   By default if this parameter is not set it will be set to 
*                   strDefaultStrip.  This will strip all white space.
*/
void CStringUtils::Trim( string& str, const string& strStrip )
{
	if( str.empty()) return; //nothing to trim

	//find first character of substring
	const string::size_type nLeftStart = str.find_first_not_of(strStrip);
	
	//might be that the string is made entirely of strip characters
	if( nLeftStart >= str.length())
{
		str.clear(); //trim it all!
		return;
}

	//find last character of substring
	const string::size_type nRightEnd = str.find_last_not_of(strStrip);

	//make a substring
	str.substr( nLeftStart, nRightEnd + 1 ).swap( str );
}


/**
* This method copies a C-string, stopping at '\0' or when the max length
* has been reached.
*
* @param rhs A void pointer with the data to be copied.
* @param nMaxStrLen A size_type indicating the number of characters to copy.
*
* @return A std:string with the copied data.
*
* @notes This method will copy all data until either a '\0' is reached
*        or nMaxStrLen characters have been copied.
*/
std::string CStringUtils::Copy( const void* pvRhs, size_type nMaxStrLen )
{
	std::string lhs;

	// Did we receive a null pointer?
	if( NULL == pvRhs )
	{
		// If so, get out because we have nothing to read.
		return lhs;
	}

	// Start at rhs and go until EITHER a '\0' is reached or until
	// nMaxStrLen characters have been copied
	const char* rhs = static_cast<const char*>( pvRhs );

	while( nMaxStrLen > 0 && *rhs != '\0' )
	{
		lhs += *rhs;
		++rhs;
		--nMaxStrLen;
	}

	return lhs;
}


/**
* This method copies a C-string, stopping when a '/0' is reached or
* at the ending location passed in.
*
* @param pvRhs A void pointer with the data to be copied.
* @param pEnd A void pointer indicating the ending location of the data to be copied.
*
* @return A std:string with the copied data.
*/
std::string CStringUtils::Copy( const void* pvRhs, const void* const pEnd )
{
	std::string lhs;

	// Did we receive a null pointer where we are supposed to start copying from?
	if( NULL == pvRhs )
	{
		// If so, just return because we have nothing to copy.
		return lhs;
	}

	// Start at rhs and go until EITHER pEnd is reached or a '\0' is reached.
	// *pEnd is NOT added to the string, i.e., it points to one-passed the end of rhs
	const char* rhs = static_cast<const char*>( pvRhs );

	while( rhs != pEnd && *rhs != '\0' )
	{
		lhs += *rhs;
		++rhs;
	}

	return lhs;
}


/**
* This method right justifies a string, padding the left most characters
* with the padding character specified.
* 
* @param strIn - The input string.
* @param nWidth - The length of the string that we want returned.  
* @param cPad - The pad character.  
*
* @return A std::string of size nWidth, with the rightmost (least significant)
*         nWidth characters from strIn copied into the return value, padded
*         with cPad on the left if necessary
*/
std::string CStringUtils::RightJustify( const std::string& strIn, size_type nWidth, char cPad )
{
	const size_type nInLen = strIn.length();

	if( nInLen == nWidth ) return strIn; //input already good-to-go

	if( nInLen < nWidth )
	{
		//input too short, need some prefix padding
		const string sPrefix( nWidth - nInLen, cPad );
		return sPrefix + strIn;
	}

	//input string is too long, needs trimmed.
	//trim from the left as if the string were right justified in a field and the left spilled
	//over and was cut off.
	return strIn.substr( nInLen - nWidth, nInLen );
}


/** 
* This method left justifies a string, padding the right most characters
* with the padding character specified.
*
* @param strIn - The input string.
* @param nWidth - The length of the string that we want returned.  
* @param cPad - The pad character.  
*
* @return A std::string of size nWidth, with the leftmost (most significant)
*         nWidth characters from strIn copied into the return value, padded
*         with cPad on the right if necessary
*/
std::string CStringUtils::LeftJustify( const std::string& strIn, size_type nWidth, char cPad )
{
	// String has a function to do this; really this is just here for completeness
	string x( strIn );

	x.resize( nWidth, cPad );

	return x;
}


/**
* Convert the string to lower case.
* 
* @param strIn - The string to convert.
*
* @return The lower case string.
*/
std::string CStringUtils::ToLower( const std::string& strIn)
{
	// Make a local copy so we do not change the passed in string
	std::string strLower = strIn;

	std::transform( strLower.begin(), strLower.end(), strLower.begin(), tolower );

	return strLower;
}


/** 
* Conver the string to upper case.
* 
* @param strIn - The string to convert.
*
* @return The lower case string.
*/
std::string CStringUtils::ToUpper( const std::string& strIn)
{
	// Make a local copy so we do not change the passed in string
	std::string strLower = strIn;

	std::transform( strLower.begin(), strLower.end(), strLower.begin(), toupper );

	return strLower;
}



/*@{
 * Wide strings are not universally available so we wrap them for conditional compilation
 *@}
 */
#if defined(HAS_WSTRING) && !defined(HAS_STDC_SECURE)
#endif

#if defined(HAS_WSTRING)

const wstring CStringUtils::wstrDefaultStrip = wstring( 1, L'\0' ) + L" \t\r\n\v\f";


/**
* This will trim the left side of a string.
*
* @param strInput - This is the input wstring to trim.  
* @param strStrip - This is a parameter that describes the characters to strip.
*                   By default if this parameter is not set it will be set to 
*                   strDefaultStrip.  This will strip all white space.
*
* @return A std::wstring containing the original string with the left side
*         stripped.
*/
wstring CStringUtils::TrimLeft(const wstring& strInput, const wstring& strStrip)
{
	// Set the initial value
	wstring strReturn = strInput;

	// Trim the left side
	strReturn.erase(0, strReturn.find_first_not_of(strStrip));
	
	// Return the trimmed string
	return strReturn;
}


/**
* This will trim the left side of a wstring.
*
* @param strInput - This is the input wstring to trim.  
* @param strStrip - This is a parameter that describes the characters to strip.
*                   By default if this parameter is not set it will be set to 
*                   strDefaultStrip.  This will strip all white space.
*
* @return A std::wstring containing the original string with the right side
*         stripped.
*/
wstring CStringUtils::TrimRight(const wstring& strInput, const wstring& strStrip)
{
	// Set the initial value
	wstring strReturn = strInput;

	// Trim the right side
	strReturn.erase(strReturn.find_last_not_of(strStrip) + 1);

	// Return the trimmed string
	return strReturn;
}


/**
* This will trim the left side of a string.
*
* @param strInput - This is the input wstring to trim.  
* @param strStrip - This is a parameter that describes the characters to strip.
*                   By default if this parameter is not set it will be set to 
*                   strDefaultStrip.  This will strip all white space.
*
* @return A std::wstring containing the original string with the left and
*         right side stripped.
*/
wstring CStringUtils::Trim(const wstring& strInput, const wstring& strStrip)
{
	// Set the initial value
	wstring strReturn = strInput;

	// Trim the left side
	strReturn.erase(strReturn.find_last_not_of(strStrip) + 1);
	
	// Trim the right side
	strReturn.erase(0, strReturn.find_first_not_of(strStrip));

	// Return the trimmed string
	return strReturn;
}


/**
* Converts a narrow string to a wide string.
*
* @param strIn - the input narrow string.
* @param strOut - the output wide string.
*/
void CStringUtils::Widen( wstring& strOut, const string& strIn )
{
	Widen( strOut, strIn.c_str(), strIn.size());
}


/**
* Converts a narrow string to a wide string.
*
* @param strIn The input narrow string.
* @param strOut The output wide string.
* @param nInLen Indicates the size of the input buffer.
*/
void CStringUtils::Widen( wstring& strOut, const char* strIn, size_type nInLen )
{
#if defined UNDER_CE
	int nResult = 0;

	// get the lenght of the string
	int nLen = MultiByteToWideChar(CP_ACP, 0, strIn, -1, NULL, NULL);

	// allocate space for the wide character string
	wchar_t* pcOut = new wchar_t[nLen+1];

    // Covert to Unicode.
	nResult = MultiByteToWideChar(CP_ACP, 0,  strIn, -1, pcOut, nLen);	

	if (nResult == 0)
	{
		strOut = L"";
	}
	else
	{
		strOut = pcOut;
	}

	delete[] pcOut;
#else
	vector<wchar_t> vecOut( nInLen + 1, L'\0' ); //+1 for terminating '\0'

	size_type nCharsConverted = 0;

	// Purposely ignoring the error value returned.  The possible error conditions
	// are handled implicitly.
	mbstowcs_s( &nCharsConverted, &vecOut[0], vecOut.size(), strIn, nInLen );

	//doing this assignment directly to strOut causes the S2 to go into alarm
	std::wstring strTemp = static_cast<wchar_t*>(&vecOut[0]);
	strOut = strTemp;
#endif
}


/**
 * Converts a wide string to a narrow string.
 *
 * @param strIn - the input wide string.
 * @param strOut - the output narrow string.
 */
void CStringUtils::Narrow( string& strOut, const wstring& strIn )
{
	Narrow( strOut, strIn.c_str(), strIn.size());
}


void CStringUtils::Narrow( string& strOut, const wchar_t* strIn, size_type nInLen )
{
#if defined UNDER_CE
	int nResult = 0;
	int nLen    = 0;

	// Get the length of the string
	nLen = WideCharToMultiByte(CP_ACP, 0, strIn, -1, NULL, NULL, NULL, NULL);

	// Allocate space for the wide character string
	char* pcOut = new char[nLen+1];

    // Covert to Unicode.
	nResult = WideCharToMultiByte(CP_ACP, 0,  strIn, -1, pcOut, nLen, NULL, NULL);	

	if (nResult == 0)
	{
		strOut = "";
	}
	else
	{
		strOut = pcOut;
	}

	delete[] pcOut;

#else
	vector<char> vecOut( nInLen + 1, '\0' ); //+1 for terminating '\0'
	size_type nCharsConverted = 0;

	// Purposely ignoring the error value returned.  The possible error conditions
	// are handled implicitly.
	wcstombs_s( &nCharsConverted, &vecOut[0], vecOut.size(), strIn, nInLen );

	//doing this assignment directly to strOut causes the S2 to go into alarm
	std::string strTemp = static_cast<char*>(&vecOut[0]);
	strOut = strTemp;
#endif
}	


/**
* Converts a narrow string to a wide string.
*
* @param strIn - the input narrow string.
*
* @return A std::wstring with the data from the input narrow string
*/
std::wstring CStringUtils::Widen( const std::string& strIn )
{
	std::wstring strOut;
	Widen( strOut, strIn );
	return strOut;
}


/**
* Converts a narrow string to a wide string.
*
* @param strIn - A char pointer to the buffer holding the data.
* @param nInLen - the length of the input narrow string.
*
* @return A std::wstring with the data from the input narrow string
*/
std::wstring CStringUtils::Widen( const char* strIn, size_type nInLen )
{
	std::wstring strOut;
	Widen( strOut, strIn, nInLen );
	return strOut;
}


/**
* Converts a wide string to a narrow string.
*
* @param strIn - the input wide string.
*
* @return A std::string with the data from the input string
*/
std::string CStringUtils::Narrow( const std::wstring& strIn )
{
	std::string strOut;

	Narrow( strOut, strIn );

	return strOut;
}


/**
* Converts a wide string to a narrow string.
*
* @param strIn - the input wide string.
* @param nInLen - the length of the input string.
*
* @return A std::string with the data from the input string
*/
std::string CStringUtils::Narrow( const wchar_t* strIn, size_type nInLen )
{
	std::string strOut;

	Narrow( strOut, strIn, nInLen );

	return strOut;
}


/** 
* Conver the string to lower case.
* 
* @param strIn - The string to convert.
*
* @return The lower case string.
*/
 std::wstring CStringUtils::ToLower( const std::wstring& strIn)
{
	// make a local copy so we do not change the passed in string
	std::wstring strLower = strIn;

	// make the string lower case
	std::transform( strLower.begin(), strLower.end(), strLower.begin(), tolower );
	
	// return the lower case string
	return strLower;
}


/** 
* Conver the string to upper case.
* 
* @param strIn - The string to convert.
*
* @return The lower case string.
*/
 std::wstring CStringUtils::ToUpper( const std::wstring& strIn)
{
	// Make a local copy so we do not change the passed in string
	std::wstring strLower = strIn;

	// make the string lower case
	std::transform( strLower.begin(), strLower.end(), strLower.begin(), toupper );
	
	// return the lower case string
	return strLower;
}



#endif //test for wstring


