/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file CheckedCopy.h
 * Header file for a copy method that checks the destination buffer before the
 * copy is performed to ensure it is large enough to hold the source and is
 * portable.
 */


///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////

#pragma once


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////

#include <stdexcept>



/**
 * Copies [src, src_end) to [dest, dest_end).
 * Same as the standard C++ copy algorithm except that the destination buffer is checked
 * to make sure it's large enough to hold the source.  Microsoft offers checked extentions,
 * but they're not portable.  This is.
 *
 * Note that it throws std::out_of_range in order to stay consistent w/ standard C++ 
 * iterators, containers & algorithms.
 * 
 * @param src Iterator to start of source
 * @param src_end Iterator to 1-passed-end of source
 * @param dest Iterator to start of destination
 * @param dest_end Iterator to 1-passed-end of destination
 * @return Iterator into destination buffer immediately after the last element copied, i.e.,
 *         it's the iterator at (dest + src_end - src).
 */
template<class In, class Out>
inline Out CheckedCopy( In src, In src_end, Out dest, Out dest_end )
{
	if((dest_end - dest) < (src_end - src))
	{
		throw (std::out_of_range)"Destination not large enough to hold copy";
	}
	while( src != src_end )
	{
		*dest++ = *src++;
	}
	return dest;
}
