/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file CompileTimeCheck.h
 * Header file for a helper function that will perform a compile-time check
 * on any condition desired. If the condition resolves to false, a compile
 * time error will be raised.
 */


///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////

#pragma once


/**
* From Modern C++ Design by Andrei Alexandrescu
*
* The idea is to generate a useful compile-time error for conditions that resolve 
* to false, rather than relying on run-time checks.  This'll work for any checking 
* of values that are constant at compile time.
* 
* In the event the condition being tested resolves to true the code will compile away
* cleanly, leaving no sticky residue.
* 
* Among other things, good for checking portibility issues, e.g., that an int really 
* is 4 bytes wide - something that's relied on heavily but not guaranteed.
* 
* Use the macro STATIC_CHECK( expr, msg ) to evaluate the expression expr.  msg
* must be a valid C/C++ identifier and is used in the error message.  Also, nest the check
* inside a code block someplace, like a function set asside for this kind of thing.
* 
* E.g., 
* 
*   static void CheckTypeAssumptions()
*   {
*     STATIC_CHECK( sizeof(int) == 4, Size_Of_Int_Not_4_Bytes );
*   }
* 
* You'd get an error like this under VC2005:
* 
*   error C2440: '<function-style-cast>' : cannot convert from 'CheckTypeAssumptions::ERROR_Size_Of_Int_Not_4_Bytes' 
*   to 'CompileTimeCheck<false>'
* 
* ...if it were actually false that is.
*/

template<bool> struct CompileTimeCheck
{
	CompileTimeCheck(...);
};

template<> struct CompileTimeCheck<false> {};

#define STATIC_CHECK( expr, msg ) \
	{\
		class ERROR_##msg {} x;\
		(void)sizeof(\
			CompileTimeCheck<(expr) != 0>( x ));\
	}

