/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only. Not withstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file SafeCheck.h
 * Header file containing templates that will keep "if-throw" blocks shorter.
 */

///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////
#pragma once

/**
*
* Convenience template to keep ``if-throw'' blocks short(er).
* 
* Usage:
* 
*   SafeCheck< X >( B, p1, p2, ... );
* 
* Where:
* 
*   X  = the exception type to be thrown if the test fails
*   B  = a boolean expression
*   pn = nth parameter passed to X's constructor
* 
* Example:
* 
*   SafeCheck<ApexException>(                   // throw a ApexException
* 	  vecData.size() > MAX_DS101_PACKET_SIZE,   // if this test fails
* 	  ERROR_FATAL,                              // p1 to ApexException
* 	  L"input size is too large",               // p2 to ApexException
* 	  __FILE__,                                 // p3 to ApexException
* 	  __LINE__ );                               // p4 to ApexException
* 
* Which would be written more shortly as:
* 
*   SafeCheck<ApexException>( vecData.size() > MAX_DS101_PACKET_SIZE,
* 	 ERROR_FATAL, L"input size is too large", __FILE__, __LINE__ );
*/

template<typename X, typename B>
inline void SafeCheck( const B& test )
{
	if( !test ) throw X();
}


template<typename X, typename B, typename T1>
inline void SafeCheck( const B& test, const T1& p1 )
{
	if( !test ) throw X( p1 );
}


template<typename X, typename B, typename T1, typename T2>
inline void SafeCheck( const B& test, const T1& p1, const T2& p2 )
{
	if( !test ) throw X( p1, p2 );
}


template<typename X, typename B, typename T1, typename T2, typename T3>
inline void SafeCheck( const B& test, const T1& p1, const T2& p2, const T3& p3 )
{
	if( !test ) throw X( p1, p2, p3 );
}


template<typename X, typename B, typename T1, typename T2, typename T3, typename T4>
inline void SafeCheck( const B& test, const T1& p1, const T2& p2, const T3& p3, const T4& p4 )
{
	if( !test ) throw X( p1, p2, p3, p4 );
}


template<typename X, typename B, typename T1, typename T2, typename T3, typename T4, typename T5>
inline void SafeCheck( const B& test, const T1& p1, const T2& p2, const T3& p3, const T4& p4, const T5& p5 )
{
	if( !test ) throw X( p1, p2, p3, p4, p5 );
}





