﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using UPhilSharp;
using UPhilSharp.Managers;
using UPhilSharp.Models;

namespace UPhilSharp_Example.ViewModel
{
    public class MainWindowViewModel : ViewModelBase
    {
        #region Member Variables

        private string loadedMessage = "UPhil loaded successfully!";
        private string comPortsMessage = string.Empty;
        private UPhilLoader uPhilLoader = new UPhilLoader();
        private Brush greenBrush = new SolidColorBrush(Colors.Green);
        private Brush redBrush = new SolidColorBrush(Colors.Red);
        private Brush loadedColor;
        private bool loadEnabled = false;
        private Visibility loadedMessageVisibility = Visibility.Collapsed;
        private int easyDS101Count = 0;
        private int uPhilUninitializedCount = 0;
        private int uPhilInitializedCount = 0;
        private EasyDS101Mgr easyDs101Mgr = new EasyDS101Mgr();
        private UPhilMgr uPhilMgr = new UPhilMgr();


        #endregion

        #region Properties

        public string LoadedMessage
        {
            get => loadedMessage;
            set
            {
                if (value != loadedMessage)
                {
                    loadedMessage = value;

                    OnPropertyChanged();
                }
            }
        }

        public Brush LoadedColor
        {
            get => loadedColor;
            set
            {
                if (value != loadedColor)
                {
                    loadedColor = value;

                    OnPropertyChanged();
                }
            }
        }

        public bool LoadEnabled
        {
            get => loadEnabled;
            set
            {
                if (value != loadEnabled)
                {
                    loadEnabled = value;

                    OnPropertyChanged();
                }
            }
        }

        public Visibility LoadedMessageVisibility
        {
            get => loadedMessageVisibility;
            set
            {
                if (value != loadedMessageVisibility)
                {
                    loadedMessageVisibility = value;

                    OnPropertyChanged();
                }
            }
        }

        public int EasyDS101Count
        {
            get => easyDS101Count;
            set
            {
                if (value != easyDS101Count)
                {
                    easyDS101Count = value;

                    OnPropertyChanged();
                }
            }
        }

        public int UPhilUninitializedCount
        {
            get => uPhilUninitializedCount;
            set
            {
                if (value != uPhilUninitializedCount)
                {
                    uPhilUninitializedCount = value;

                    LoadEnabled = uPhilUninitializedCount > 0;

                    OnPropertyChanged();
                }
            }
        }

        public int UPhilInitializedCount
        {
            get => uPhilInitializedCount;
            set
            {
                if (value != uPhilInitializedCount)
                {
                    uPhilInitializedCount = value;

                    OnPropertyChanged();

                    COMPortsMessage = CalculateComPortMessage();
                }
            }
        }

        public string COMPortsMessage
        {
            get => comPortsMessage;
            set
            {
                if (value != comPortsMessage)
                {
                    comPortsMessage = value;

                    OnPropertyChanged();
                }
            }
        }

        #endregion

        #region Constructor(s)

        public MainWindowViewModel()
        {
        }

        #endregion

        #region Public Methods

        public void InitWindowAfterLoaded()
        {
            easyDs101Mgr.EasyDeviceInserted += EasyDeviceInserted;
            easyDs101Mgr.EasyDeviceRemoved += EasyDeviceRemoved;

            EasyDS101Count = easyDs101Mgr.EasyDs101Devices.Count;

            uPhilMgr.UninitializedUPhilInserted += UninitializedUPhilInserted;
            uPhilMgr.UninitializedUPhilRemoved += UninitializedUPhilRemoved;
            uPhilMgr.InitializedUPhilInserted += InitializedUPhilInserted;
            uPhilMgr.InitializedUPhilRemoved += InitializedUPhilRemoved;

            UPhilUninitializedCount = uPhilMgr.UninitializedDevices.Count;
            UPhilInitializedCount = uPhilMgr.InitializedDevices.Count;
        }

        #endregion

        #region ICommands

        public ICommand LoadCmd { get { return new RelayCommand(arg => LoadUPhil()); } }


        #endregion

        #region Private Methods

        private string CalculateComPortMessage()
        {
            if (uPhilMgr.InitializedDevices.Count > 0)
            {
                string msg = "UPhil(s) found on: ";

                foreach (var dev in uPhilMgr.InitializedDevices)
                {
                    msg += dev.ComPortName + ";";
                }

                msg = msg.Remove(msg.Length - 1, 1);

                return msg;
            }

            return string.Empty;
        }

        private async void LoadUPhil()
        {
            LoadEnabled = false;

            int nReturnCode = await uPhilLoader.LoadUPhilFirmwareAsync();

            if (nReturnCode == 0)
            {
                LoadedMessage = "UPhil loaded successfully!";
                LoadedColor = greenBrush;
            }
            else
            {
                LoadedMessage = "UPhil load failed!";
                LoadedColor = redBrush;
                LoadEnabled = true;
            }

            LoadedMessageVisibility = Visibility.Visible;
        }

        private void UninitializedUPhilInserted()
        {
            UPhilUninitializedCount = uPhilMgr.UninitializedDevices.Count;
            UPhilInitializedCount = uPhilMgr.InitializedDevices.Count;

            LoadEnabled = UPhilUninitializedCount > 0;
        }

        private void UninitializedUPhilRemoved()
        {
            UPhilUninitializedCount = uPhilMgr.UninitializedDevices.Count;
            UPhilInitializedCount = uPhilMgr.InitializedDevices.Count;

            LoadEnabled = UPhilUninitializedCount > 0;
        }

        private void InitializedUPhilInserted(UPhilDevice uPhilDevice)
        {
            UPhilUninitializedCount = uPhilMgr.UninitializedDevices.Count;
            UPhilInitializedCount = uPhilMgr.InitializedDevices.Count;

            LoadEnabled = UPhilUninitializedCount > 0;
        }

        private void InitializedUPhilRemoved(UPhilDevice uPhilDevice)
        {
            UPhilUninitializedCount = uPhilMgr.UninitializedDevices.Count;
            UPhilInitializedCount = uPhilMgr.InitializedDevices.Count;

            LoadEnabled = UPhilUninitializedCount > 0;
        }

        private void EasyDeviceRemoved(EasyDS101Device easyDevice)
        {
            EasyDS101Count = easyDs101Mgr.EasyDs101Devices.Count;
        }

        private void EasyDeviceInserted(EasyDS101Device easyDevice)
        {
            EasyDS101Count = easyDs101Mgr.EasyDs101Devices.Count;
        }

        #endregion
    }
}
